!     ------------------------------------------------------------------
!
!     main.f95
!
!
!     Created on Sun Apr 08 16:07:51 2007
!     Copyright (c) 2007 James K Beard. All rights reserved.
!
!
!     ------------------------------------------------------------------

program main	!Generate v-i curve family for IGFET
use MOSFETs,only:VTransition,V_DS_Early,K_conductance_parameter,generate_vi_csv
implicit none
!Elementary model
!Triode region model is ID = K*(2*(VG-VTransistion)*VD - VD^2)
!Constant current model is ID = K*(VG-VTransistion)^2+VD/RDrain
!=IF($A3>C$1-VTransistion,KConst*(C$1-VTransistion)^2+$A3/RDrain,KConst*(2*(C$1-VTransistion)*$A3-$A3^2))
!The transition between these is when VD = VG - VTransistion, or ID = K*VD^2
!The intersection of these lines is always included in the output table

!------------------------------
!contains subroutine generate_vi_csv(lunout,fname,VGDelta,NVGate,VDDelta,VDMax)
integer::NVGate
real(kind(0d0))::VGDelta,VDDelta,VDMax
character(len=40)::fname="MOSFET_transistor_vi",input_file="MOSFET_input_data.txt",rmf="ReadMe2Use.txt"
integer,parameter::lunout=10,lundataf=11,lunreadme=12
logical::already_called
!real(kind(0.d0))::VDrain,VDIntercept1,VDInterceptN,VDDelta_start


  inquire(file=trim(input_file),exist=already_called)
  if(.NOT.already_called) then
!Define transistor parameters
    VTransition=2.d0		!Turn-on at 2 Volts
    V_DS_Early=100		!100 Volts
    K_conductance_parameter=0.8d-3

!Define plotting paramters

    VGDelta=1.d0	!1 V
    NVGate=6	!2 to 8 V

    VDDelta=1.d0	!1 Volt increments
    VDmax=20d0	!0 to 20 Volts
    open(lundataf,file=trim(input_file))
    write(lundataf,1000) VTransition,V_DS_Early,	&
      K_conductance_parameter,VGDelta,NVGate,VDDelta,VDmax
    close(lundataf)
    open(lunreadme,file=trim(rmf))
    write(lunreadme,3000) trim(input_file),trim(input_file),trim(fname)
    close(lunreadme)
  else
    open(lundataf,file=trim(input_file))
    read(lundataf,2000) VTransition,V_DS_Early,	&
      K_conductance_parameter,VGDelta,NVGate,VDDelta,VDmax
    close(lundataf)
  end if

  call generate_vi_csv(lunout,fname,VGDelta,NVGate,VDDelta,VDMax)

  print *,"Normal exit."

1000 format("Data file for MOSFET model and v-i courve plotter"/	&
  g25.15,",VTransition, Volts; typically 2 Volts"/	&
  g25.15,",V_DS_Early, Volts; typically 100 Volts"/	&
  g25.15,",K_conductance_parameter, Amp/Volt^2; from 8 X 10^(-4) to 1"/	&
  g25.15,",VGDelta, Volts; difference in gate voltage for v-i plots"/	&
  I25,",NVGate, number of v-i plots; VG = i*VGDelta+VTransistion, i=0,NVGate"/	&
  g25.15,",VDDelta, Volts; plot granularity in constant current region"/	&
  g25.15,",VDmax, Volts; maximum drain voltage for v-i plots"/)
2000 format(/4(g25.15/),i25/2(g25.15/))
3000 format("Program copyright 2007 by James K Beard."/				&
  "There are no warranties on this program, express or implied."//		&
  "To use, edit the file ",a//							&
  "If the program is called and ",a," is not present, it generates a new"/	&
  "one with default parameters."//						&
  "The first three parameters determine the transistor characteristics."/	&
  "The next two parameters determine the range of gate voltages used."/		&
  "The last two parameters determine the maximum drain voltage plotted"/	&
  "and the granularity of the data in the constant current region.  The"/	&
  "program automatically adjusts the granularity for the triode region."//	&
  "To plot the v-i curves, load the output file ",a,".csv"/			&
  "into a spreadsheet program.  Click on the Chart Wizard, select chart"/	&
  "type ""XY (scatter)"", of type ""data points connected by lines"/		&
  "without markers."""/)

end program main
